#########################################################################
# form.tcl - Make interfaces to procs that follow certain conventions.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
######################################################################

proc form {w labels proc {list 1}} {
    frame $w -class Form
    frame $w.sel
    pack [listbox $w.sel.list -yscroll [list $w.sel.yscroll set]] -side left
    pack [scrollbar $w.sel.yscroll -command [list $w.sel.list yview]] \
	      -side left -fill y

    frame $w.edit
    frame $w.edit.buts
    pack [button $w.edit.buts.add -text Add \
	      -command [list form::add $w $proc]] -side left
    pack [button $w.edit.buts.delete -text Delete \
	      -command [list form::delete $w $proc]] -side left
    frame $w.edit.f
    set count 0
    foreach label $labels {
	grid [label $w.edit.f.l$count -text $label] -sticky w -column 0
	grid [entry $w.edit.f.e$count] -row $count -sticky ew -column 1
	incr count
    }
    
    pack $w.edit.f $w.edit.buts -side top
    pack $w.sel $w.edit -side left
   
    bind $w.edit.f.e[incr count -1] <Key-Return> [list form::add $w $proc]
    bind $w.sel.list <ButtonRelease-1> \
	[list form::show_selection $w $proc $list]

    form::listbox_update $w $proc

    return $w
}

namespace eval form {}

proc form::show_selection {w proc {list 1}} {
    set i [$w.sel.list curselection]

    if {$i != ""} {
	set i [$w.sel.list get [$w.sel.list curselection]]
	if {$list} {
	    set_entries $w [concat [list $i] [$proc set $i]]
	} else {
	    set_entries $w [list $i [$proc set $i]]
	}
    }

    return
}

proc form::add {w proc} {
    eval {$proc} set [get_entries $w]
    set_entries $w {}
    listbox_update $w $proc

    return
}

proc form::delete {w proc} {
    set i [$w.sel.list curselection]

    if {$i != ""} {
	set i [$w.sel.list get $i]
	
	# Mask of string match pattern
	regsub -all {[\[*?]} $i {\\&} i

	$proc delete $i
	set_entries $w {}
	listbox_update $w $proc
    }

    return
}

proc form::listbox_update {w proc} {
    $w.sel.list delete 0 end
    foreach i [lsort [$proc names]] {
        $w.sel.list insert end $i
    }

    return
}

proc form::get_entries {w} {
    set num_entries [expr {[llength [winfo children $w.edit.f]]/2}]

    for {set i 0} {$i < $num_entries} {incr i} {
	lappend list [$w.edit.f.e$i get]
    }
    return $list
}

proc form::set_entries {w list} {
    set num_entries [expr {[llength [winfo children $w.edit.f]]/2}]

    for {set i 0} {$i < $num_entries} {incr i} {
	set str [lindex $list $i]
 	$w.edit.f.e$i delete 0 end
 	$w.edit.f.e$i insert end $str
    }

    return
}